@@ -370,15 +370,30 @@ codecs_to_string(GSList *codecs)
 				       c->clock_rate);
 
 		if (params) {
-			g_string_append_printf(result, "a=fmtp:%d", c->id);
+			GString *param_str = g_string_new(NULL);
+			int written_params = 0;
+
+			g_string_append_printf(param_str, "a=fmtp:%d", c->id);
 
 			for (; params; params = params->next) {
 				struct sipnameval* par = params->data;
-				g_string_append_printf(result, " %s=%s",
+				if (sipe_strequal(par->name, "farsight-send-profile")) {
+					// Lync AVMCU doesn't like this property.
+					continue;
+				}
+
+				g_string_append_printf(param_str, " %s=%s",
 						       par->name, par->value);
+				++written_params;
 			}
 
-			g_string_append(result, "\r\n");
+			g_string_append(param_str, "\r\n");
+
+			if (written_params > 0) {
+				g_string_append(result, param_str->str);
+			}
+
+			g_string_free(param_str, TRUE);
 		}
 	}
 
@@ -414,19 +429,6 @@ base64_unpad(const gchar *str)
 	return result;
 }
 
-static gint
-candidate_sort_cb(struct sdpcandidate *c1, struct sdpcandidate *c2)
-{
-	int cmp = sipe_strcompare(c1->foundation, c2->foundation);
-	if (cmp == 0) {
-		cmp = sipe_strcompare(c1->username, c2->username);
-		if (cmp == 0)
-			cmp = c1->component - c2->component;
-	}
-
-	return cmp;
-}
-
 static gchar *
 candidates_to_string(GSList *candidates, SipeIceVersion ice_version)
 {
@@ -434,9 +436,6 @@ candidates_to_string(GSList *candidates, SipeIceVersion ice_version)
 	GSList *i;
 	GSList *processed_tcp_candidates = NULL;
 
-	candidates = g_slist_copy(candidates);
-	candidates = g_slist_sort(candidates, (GCompareFunc)candidate_sort_cb);
-
 	for (i = candidates; i; i = i->next) {
 		struct sdpcandidate *c = i->data;
 		const gchar *protocol;
@@ -468,8 +467,8 @@ candidates_to_string(GSList *candidates, SipeIceVersion ice_version)
 				case SIPE_CANDIDATE_TYPE_RELAY:
 					type = "relay";
 					related = g_strdup_printf("raddr %s rport %d ",
-								  c->ip,
-								  c->port);
+								  c->base_ip,
+								  c->base_port);
 					break;
 				case SIPE_CANDIDATE_TYPE_SRFLX:
 					type = "srflx";
@@ -555,7 +554,6 @@ candidates_to_string(GSList *candidates, SipeIceVersion ice_version)
 		}
 	}
 
-	g_slist_free(candidates);
 	g_slist_free(processed_tcp_candidates);
 
 	return g_string_free(result, FALSE);
@@ -566,9 +564,6 @@ remote_candidates_to_string(GSList *candidates, SipeIceVersion ice_version)
 {
 	GString *result = g_string_new("");
 
-	candidates = g_slist_copy(candidates);
-	candidates = g_slist_sort(candidates, (GCompareFunc)candidate_sort_cb);
-
 	if (candidates) {
 		if (ice_version == SIPE_ICE_RFC_5245) {
 			GSList *i;
@@ -588,8 +583,6 @@ remote_candidates_to_string(GSList *candidates, SipeIceVersion ice_version)
 		}
 	}
 
-	g_slist_free(candidates);
-
 	return g_string_free(result, FALSE);
 }
 
@@ -754,7 +747,7 @@ sdpcandidate_free(struct sdpcandidate *candidate)
 	}
 }
 
-static void
+void
 sdpcodec_free(struct sdpcodec *codec)
 {
 	if (codec) {
