@@ -169,32 +169,30 @@ sipe_cal_events_free(GSList *cal_events)
 void
 sipe_cal_calendar_free(struct sipe_calendar *cal)
 {
-	g_free(cal->email);
-	g_free(cal->legacy_dn);
-	g_free(cal->auth_domain);
-	g_free(cal->auth_user);
-	g_free(cal->password);
-	g_free(cal->as_url);
-	g_free(cal->oof_url);
-	g_free(cal->oab_url);
-	g_free(cal->domino_url);
-	g_free(cal->oof_state);
-	g_free(cal->oof_note);
-	g_free(cal->free_busy);
-	g_free(cal->working_hours_xml_str);
+	if (cal) {
+		g_free(cal->email);
+		g_free(cal->legacy_dn);
+		g_free(cal->as_url);
+		g_free(cal->oof_url);
+		g_free(cal->oab_url);
+		g_free(cal->domino_url);
+		g_free(cal->oof_state);
+		g_free(cal->oof_note);
+		g_free(cal->free_busy);
+		g_free(cal->working_hours_xml_str);
 
-	sipe_cal_events_free(cal->cal_events);
+		sipe_cal_events_free(cal->cal_events);
 
-	if (cal->request)
-		sipe_http_request_cancel(cal->request);
-	sipe_http_session_close(cal->session);
+		if (cal->request)
+			sipe_http_request_cancel(cal->request);
+		sipe_http_session_close(cal->session);
 
-	g_free(cal);
+		g_free(cal);
+	}
 }
 
-gboolean
-sipe_cal_calendar_init(struct sipe_core_private *sipe_private,
-		       gboolean *has_url)
+void
+sipe_cal_calendar_init(struct sipe_core_private *sipe_private)
 {
 	if (!sipe_private->calendar) {
 		struct sipe_calendar *cal;
@@ -207,29 +205,13 @@ sipe_cal_calendar_init(struct sipe_core_private *sipe_private,
 
 		/* user specified a service URL? */
 		value = sipe_backend_setting(SIPE_CORE_PUBLIC, SIPE_SETTING_EMAIL_URL);
-		if (has_url) *has_url = !is_empty(value);
 		if (!is_empty(value)) {
 			cal->as_url  = g_strdup(value);
 			cal->oof_url = g_strdup(value);
 			cal->domino_url  = g_strdup(value);
 		}
-
-		/* user specified email login? */
-		value = sipe_backend_setting(SIPE_CORE_PUBLIC, SIPE_SETTING_EMAIL_LOGIN);
-		if (!is_empty(value)) {
-			/* Allowed domain-account separators are / or \ */
-			gchar **domain_user = g_strsplit_set(value, "/\\", 2);
-			gboolean has_domain = domain_user[1] != NULL;
-
-			cal->auth_domain = has_domain ? g_strdup(domain_user[0]) : NULL;
-			cal->auth_user   = g_strdup(domain_user[has_domain ? 1 : 0]);
-			cal->password    = g_strdup(sipe_backend_setting(SIPE_CORE_PUBLIC,
-									 SIPE_SETTING_EMAIL_PASSWORD));
-			g_strfreev(domain_user);
-		}
-		return TRUE;
 	}
-	return FALSE;
+	return;
 }
 
 
@@ -1128,16 +1110,6 @@ void sipe_cal_delayed_calendar_update(struct sipe_core_private *sipe_private)
 				      NULL);
 }
 
-void sipe_cal_http_authentication(struct sipe_calendar *cal)
-{
-	if (cal->auth_user) {
-		sipe_http_request_authentication(cal->request,
-						 cal->auth_domain,
-						 cal->auth_user,
-						 cal->password);
-	}
-}
-
 /*
   Local Variables:
   mode: c
