@@ -3,7 +3,7 @@
  *
  * pidgin-sipe
  *
- * Copyright (C) 2010-2013 SIPE Project <http://sipe.sourceforge.net/>
+ * Copyright (C) 2010-2014 SIPE Project <http://sipe.sourceforge.net/>
  * Copyright (C) 2009 pier11 <pier11@operamail.com>
  *
  *
@@ -22,6 +22,15 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+/**
+ * Documentation references:
+ *
+ * Microsoft DevNet: [MS-CONFIM]: Centralized Conference Control Protocol:
+ *                                Instant Messaging Extensions
+ *  <http://msdn.microsoft.com/en-us/library/cc431500%28v=office.12%29.aspx>
+ *
+ */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -324,22 +333,20 @@ parse_lync_join_url(const gchar *uri)
 
 	for (parts_count = 0; parts[parts_count]; ++parts_count);
 	if (parts_count >= 3) {
-		gchar *base_url = parts[0];
-		gchar *conference_id = parts[parts_count - 1];
-		gchar *organizer_alias = parts[parts_count - 2];
+		const gchar *conference_id   = parts[parts_count - 1];
+		const gchar *organizer_alias = parts[parts_count - 2];
 
-		gchar **url_parts = g_strsplit(base_url, ".", 0);
-		int url_parts_count = 0;
-		for (url_parts_count = 0; url_parts[url_parts_count]; ++url_parts_count);
+		gchar **domain_parts = g_strsplit(parts[0], ".", 2);
 
-		if (url_parts_count >= 3) {
-			focus_uri = g_strdup_printf("sip:%s@%s.%s;gruu;opaque=app:conf:focus:id:%s",
-					organizer_alias,
-					url_parts[url_parts_count - 2], url_parts[url_parts_count - 1],
-					conference_id);
+		/* we need to drop the first sub-domain from the URL */
+		if (domain_parts[0] && domain_parts[1]) {
+			focus_uri = g_strdup_printf("sip:%s@%s;gruu;opaque=app:conf:focus:id:%s",
+						    organizer_alias,
+						    domain_parts[1],
+						    conference_id);
 		}
 
-		g_strfreev(url_parts);
+		g_strfreev(domain_parts);
 	}
 
 	g_strfreev(parts);
@@ -355,6 +362,10 @@ sipe_core_conf_create(struct sipe_core_public *sipe_public,
 	gchar *focus_uri;
 	struct sip_session *session = NULL;
 
+	SIPE_DEBUG_INFO("sipe_core_conf_create: URI '%s' unescaped '%s'",
+			uri    ? uri    : "<UNDEFINED>",
+			uri_ue ? uri_ue : "<UNDEFINED>");
+
 	focus_uri = parse_ocs_focus_uri(uri_ue);
 	if (!focus_uri) {
 		focus_uri = parse_lync_join_url(uri_ue);
@@ -1217,7 +1228,17 @@ sipe_process_imdn(struct sipe_core_private *sipe_private,
 	for (node = sipe_xml_child(xn_imdn, "recipient"); node; node = sipe_xml_twin(node)) {
 		gchar *tmp = parse_from(sipe_xml_attribute(node, "uri"));
 		gchar *uri = parse_from(tmp);
-		sipe_user_present_message_undelivered(sipe_private, session, -1, -1, uri, message);
+		gchar *status = sipe_xml_data(sipe_xml_child(node, "status"));
+		guint error = status ? g_ascii_strtoull(status, NULL, 10) : 0;
+		/* default to error if missing or conversion failed */
+		if ((error == 0) || (error >= 300))
+			sipe_user_present_message_undelivered(sipe_private,
+							      session,
+							      error,
+							      -1,
+							      uri,
+							      message);
+		g_free(status);
 		g_free(tmp);
 		g_free(uri);
 	}
