@@ -3,7 +3,7 @@
  *
  * pidgin-sipe
  *
- * Copyright (C) 2010-2013 SIPE Project <http://sipe.sourceforge.net/>
+ * Copyright (C) 2010-2014 SIPE Project <http://sipe.sourceforge.net/>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,11 +26,31 @@
 
 #include <glib.h>
 
-#include "blist.h"
 #include "notify.h"
-#include "privacy.h"
 #include "request.h"
+
 #include "version.h"
+#if PURPLE_VERSION_CHECK(3,0,0)
+#include "account.h"
+#include "buddylist.h"
+#else
+#include "blist.h"
+#include "privacy.h"
+#define purple_account_privacy_check(a, n)             purple_privacy_check(a, n)
+#define purple_account_privacy_deny_add(a, n, l)       purple_privacy_deny_add(a, n, l)
+#define purple_account_privacy_deny_remove(a, n, l)    purple_privacy_deny_remove(a, n, l)
+#define purple_blist_find_buddies(a, n)                purple_find_buddies(a, n)
+#define purple_blist_find_buddy(a, n)                  purple_find_buddy(a, n)
+#define purple_blist_find_buddy_in_group(a, n, g)      purple_find_buddy_in_group(a, n, g)
+#define purple_blist_find_group(n)                     purple_find_group(n)
+#define purple_buddy_set_local_alias(b, n)             purple_blist_alias_buddy(b, n)
+#define purple_buddy_set_server_alias(b, n)            purple_blist_server_alias_buddy(b, n)
+#define purple_buddy_set_name(b, n)                    purple_blist_rename_buddy(b, n)
+#define purple_group_set_name(g, n)                    purple_blist_rename_group(g, n)
+#define purple_notify_user_info_add_pair_html(i, k, v) purple_notify_user_info_add_pair(i, k, v)
+#define PURPLE_IS_BUDDY(b)                             PURPLE_BLIST_NODE_IS_BUDDY(b)
+#define PURPLE_IS_GROUP(b)                             PURPLE_BLIST_NODE_IS_GROUP(b)
+#endif
 
 #include "sipe-common.h"
 #include "sipe-backend.h"
@@ -81,16 +101,16 @@ sipe_backend_buddy sipe_backend_buddy_find(struct sipe_core_public *sipe_public,
 
 	if (group_name)
 	{
-		purple_group = purple_find_group(group_name);
+		purple_group = purple_blist_find_group(group_name);
 		if (!purple_group)
 			return NULL;
 
-		return purple_find_buddy_in_group(purple_private->account,
-						  buddy_name,
-						  purple_group);
+		return purple_blist_find_buddy_in_group(purple_private->account,
+							buddy_name,
+							purple_group);
 	} else {
-		return purple_find_buddy(purple_private->account,
-					 buddy_name);
+		return purple_blist_find_buddy(purple_private->account,
+					       buddy_name);
 	}
 }
 
@@ -106,13 +126,13 @@ GSList* sipe_backend_buddy_find_all(struct sipe_core_public *sipe_public,
 		return NULL;
 	}
 
-	return purple_find_buddies(purple_private->account, buddy_name);
+	return purple_blist_find_buddies(purple_private->account, buddy_name);
 }
 
 gchar* sipe_backend_buddy_get_name(SIPE_UNUSED_PARAMETER struct sipe_core_public *sipe_public,
 				   const sipe_backend_buddy who)
 {
-	return g_strdup(((PurpleBuddy*)who)->name);
+	return g_strdup(purple_buddy_get_name((PurpleBuddy *) who));
 }
 
 gchar* sipe_backend_buddy_get_alias(SIPE_UNUSED_PARAMETER struct sipe_core_public *sipe_public,
@@ -131,25 +151,25 @@ gchar *sipe_backend_buddy_get_local_alias(SIPE_UNUSED_PARAMETER struct sipe_core
 					  const sipe_backend_buddy who)
 {
 	return g_strdup(
-#if PURPLE_VERSION_CHECK(2,6,0) || PURPLE_VERSION_CHECK(3,0,0)
-		purple_buddy_get_local_buddy_alias
+#if PURPLE_VERSION_CHECK(2,6,0)
+		purple_buddy_get_local_buddy_alias(who)
 #else
-		purple_buddy_get_local_alias
+		purple_buddy_get_local_alias(who)
 #endif
-		(who));
+		);
 }
 
 gchar* sipe_backend_buddy_get_group_name(SIPE_UNUSED_PARAMETER struct sipe_core_public *sipe_public,
 					 const sipe_backend_buddy who)
 {
-	return g_strdup(purple_buddy_get_group((PurpleBuddy*)who)->name);
+	return g_strdup(purple_group_get_name(purple_buddy_get_group((PurpleBuddy*)who)));
 }
 
 guint sipe_backend_buddy_get_status(struct sipe_core_public *sipe_public,
 				    const gchar *uri)
 {
 	struct sipe_backend_private *purple_private = sipe_public->backend_private;
-	PurpleBuddy *pbuddy = purple_find_buddy(purple_private->account, uri);
+	PurpleBuddy *pbuddy = purple_blist_find_buddy(purple_private->account, uri);
 	const PurplePresence *presence = purple_buddy_get_presence(pbuddy);
 	const PurpleStatus *pstatus = purple_presence_get_active_status(presence);
 	return(sipe_purple_token_to_activity(purple_status_get_id(pstatus)));
@@ -159,14 +179,14 @@ void sipe_backend_buddy_set_alias(SIPE_UNUSED_PARAMETER struct sipe_core_public
 				  const sipe_backend_buddy who,
 				  const gchar *alias)
 {
-	purple_blist_alias_buddy(who, alias);
+	purple_buddy_set_local_alias(who, alias);
 }
 
 void sipe_backend_buddy_set_server_alias(SIPE_UNUSED_PARAMETER struct sipe_core_public *sipe_public,
 					 const sipe_backend_buddy who,
 					 const gchar *alias)
 {
-	purple_blist_server_alias_buddy(who, alias);
+	purple_buddy_set_server_alias(who, alias);
 }
 
 gchar* sipe_backend_buddy_get_string(SIPE_UNUSED_PARAMETER struct sipe_core_public *sipe_public,
@@ -207,7 +227,7 @@ sipe_backend_buddy sipe_backend_buddy_add(struct sipe_core_public *sipe_public,
 {
 	struct sipe_backend_private *purple_private = sipe_public->backend_private;
 	PurpleBuddy *b;
-	PurpleGroup *purple_group = purple_find_group(groupname);
+	PurpleGroup *purple_group = purple_blist_find_group(groupname);
 
 	if (!purple_group)
 		return NULL;
@@ -265,7 +285,7 @@ gboolean sipe_backend_buddy_is_blocked(struct sipe_core_public *sipe_public,
 {
 	struct sipe_backend_private *purple_private = sipe_public->backend_private;
 
-	return !purple_privacy_check(purple_private->account, who);
+	return(!purple_account_privacy_check(purple_private->account, who));
 }
 
 void sipe_backend_buddy_set_blocked_status(struct sipe_core_public *sipe_public,
@@ -275,9 +295,9 @@ void sipe_backend_buddy_set_blocked_status(struct sipe_core_public *sipe_public,
 	struct sipe_backend_private *purple_private = sipe_public->backend_private;
 
 	if (blocked) {
-		purple_privacy_deny_add(purple_private->account, who, TRUE);
+		purple_account_privacy_deny_add(purple_private->account, who, TRUE);
 	} else {
-		purple_privacy_deny_remove(purple_private->account, who, TRUE);
+		purple_account_privacy_deny_remove(purple_private->account, who, TRUE);
 	}
 
 	/* stupid workaround to make pidgin re-render screen to reflect our changes */
@@ -293,10 +313,29 @@ void sipe_backend_buddy_set_status(struct sipe_core_public *sipe_public,
 				   guint activity)
 {
 	struct sipe_backend_private *purple_private = sipe_public->backend_private;
+	PurpleBuddy *buddy = NULL;
+	PurpleStatus *status = NULL;
+	const gchar *tmp = NULL;
 
-	purple_prpl_got_user_status(purple_private->account, who,
-				    sipe_purple_activity_to_token(activity),
-				    NULL);
+	buddy = purple_blist_find_buddy(purple_private->account, who);
+	if (buddy)
+		status = purple_presence_get_active_status(purple_buddy_get_presence(buddy));
+
+	if (status)
+		tmp = sipe_core_buddy_status(PURPLE_BUDDY_TO_SIPE_CORE_PUBLIC,
+                                      purple_buddy_get_name(buddy),
+                                      sipe_purple_token_to_activity(purple_status_get_id(status)),
+                                      purple_status_get_name(status));
+
+	if (tmp)
+		purple_prpl_got_user_status(purple_private->account, who,
+					    sipe_purple_activity_to_token(activity),
+					    SIPE_PURPLE_STATUS_ATTR_ID_MESSAGE, tmp,
+					    NULL);
+	else
+		purple_prpl_got_user_status(purple_private->account, who,
+					    sipe_purple_activity_to_token(activity),
+					    NULL);
 }
 
 gboolean sipe_backend_uses_photo(void)
@@ -334,7 +373,7 @@ const gchar *sipe_backend_buddy_get_photo_hash(struct sipe_core_public *sipe_pub
 gboolean sipe_backend_buddy_group_add(SIPE_UNUSED_PARAMETER struct sipe_core_public *sipe_public,
 				      const gchar *group_name)
 {
-	PurpleGroup * purple_group = purple_find_group(group_name);
+	PurpleGroup * purple_group = purple_blist_find_group(group_name);
 	if (!purple_group) {
 		purple_group = purple_group_new(group_name);
 		purple_blist_add_group(purple_group, NULL);
@@ -347,16 +386,16 @@ gboolean sipe_backend_buddy_group_rename(SIPE_UNUSED_PARAMETER struct sipe_core_
 					 const gchar *old_name,
 					 const gchar *new_name)
 {
-	PurpleGroup *purple_group = purple_find_group(old_name);
+	PurpleGroup *purple_group = purple_blist_find_group(old_name);
 	if (purple_group)
-		purple_blist_rename_group(purple_group, new_name);
+		purple_group_set_name(purple_group, new_name);
 	return(purple_group != NULL);
 }
 
 void sipe_backend_buddy_group_remove(SIPE_UNUSED_PARAMETER struct sipe_core_public *sipe_public,
 				     const gchar *group_name)
 {
-	PurpleGroup *purple_group = purple_find_group(group_name);
+	PurpleGroup *purple_group = purple_blist_find_group(group_name);
 	if (purple_group)
 		purple_blist_remove_group(purple_group);
 }
@@ -372,14 +411,9 @@ void sipe_backend_buddy_info_add(SIPE_UNUSED_PARAMETER struct sipe_core_public *
 				 const gchar *value)
 {
 	if (info) {
-#if PURPLE_VERSION_CHECK(3,0,0)
-		purple_notify_user_info_add_pair_html
-#else
-		purple_notify_user_info_add_pair
-#endif
-			((PurpleNotifyUserInfo *) info,
-			 buddy_info_description(key),
-			 value);
+		purple_notify_user_info_add_pair_html((PurpleNotifyUserInfo *) info,
+						      buddy_info_description(key),
+						      value);
 	}
 }
 
@@ -408,12 +442,9 @@ void sipe_backend_buddy_tooltip_add(SIPE_UNUSED_PARAMETER struct sipe_core_publi
 				    const gchar *description,
 				    const gchar *value)
 {
-#if PURPLE_VERSION_CHECK(3,0,0)
-	purple_notify_user_info_add_pair_html
-#else
-	purple_notify_user_info_add_pair
-#endif
-		((PurpleNotifyUserInfo *) tooltip, description, value);
+	purple_notify_user_info_add_pair_html((PurpleNotifyUserInfo *) tooltip,
+					      description,
+					      value);
 }
 
 void sipe_purple_add_buddy(PurpleConnection *gc,
@@ -425,8 +456,8 @@ void sipe_purple_add_buddy(PurpleConnection *gc,
 	)
 {
 	SIPE_DEBUG_INFO("sipe_purple_add_buddy[CB]: buddy:%s group:%s",
-			buddy ? buddy->name : "",
-			group ? group->name : "");
+			buddy ? purple_buddy_get_name(buddy) : "",
+			group ? purple_group_get_name(group) : "");
 
 	/* libpurple can call us with undefined buddy or group */
 	if (buddy && group) {
@@ -434,23 +465,27 @@ void sipe_purple_add_buddy(PurpleConnection *gc,
 		 * Buddy name must be lower case as we use
 		 * purple_normalize_nocase() to compare
 		 */
-		gchar *buddy_name = g_ascii_strdown(buddy->name, -1);
+		gchar *buddy_name = g_ascii_strdown(purple_buddy_get_name(buddy), -1);
 		gchar *uri        = sip_uri_if_valid(buddy_name);
 		g_free(buddy_name);
 
 		if (uri) {
-			purple_blist_rename_buddy(buddy, uri);
+			purple_buddy_set_name(buddy, uri);
 			g_free(uri);
 
 			sipe_core_buddy_add(PURPLE_GC_TO_SIPE_CORE_PUBLIC,
-					    buddy->name,
-					    group->name);
+					    purple_buddy_get_name(buddy),
+					    purple_group_get_name(group));
 		} else {
 			SIPE_DEBUG_ERROR_NOFORMAT("sipe_purple_add_buddy[CB]: buddy name is invalid for URI");
 			purple_blist_remove_buddy(buddy);
 			purple_notify_error(gc, NULL,
 					    _("User name should be a valid SIP URI\nExample: user@company.com"),
-					    NULL);
+					    NULL
+#if PURPLE_VERSION_CHECK(3,0,0)
+					    , NULL
+#endif
+					    );
 		}
 	}
 }
@@ -459,12 +494,14 @@ void sipe_purple_remove_buddy(PurpleConnection *gc,
 			      PurpleBuddy *buddy,
 			      PurpleGroup *group)
 {
-	SIPE_DEBUG_INFO("sipe_purple_remove_buddy[CB]: buddy:%s group:%s", buddy ? buddy->name : "", group ? group->name : "");
+	SIPE_DEBUG_INFO("sipe_purple_remove_buddy[CB]: buddy: '%s' group: '%s'",
+			buddy ? purple_buddy_get_name(buddy) : "",
+			group ? purple_group_get_name(group) : "");
 	if (!buddy) return;
 
 	sipe_core_buddy_remove(PURPLE_GC_TO_SIPE_CORE_PUBLIC,
-			       buddy->name,
-			       group ? group->name : NULL);
+			       purple_buddy_get_name(buddy),
+			       group ? purple_group_get_name(group) : NULL);
 }
 
 void sipe_purple_group_buddy(PurpleConnection *gc,
@@ -480,46 +517,47 @@ void sipe_purple_group_buddy(PurpleConnection *gc,
 static void sipe_purple_buddy_make_chat_leader_cb(PurpleBuddy *buddy,
 						  gpointer parameter)
 {
-	SIPE_DEBUG_INFO("sipe_purple_buddy_make_chat_leader_cb: buddy->name=%s",
-			buddy->name);
+	SIPE_DEBUG_INFO("sipe_purple_buddy_make_chat_leader_cb: name '%s'",
+			purple_buddy_get_name(buddy));
 	sipe_core_conf_make_leader(PURPLE_BUDDY_TO_SIPE_CORE_PUBLIC,
 				   parameter,
-				   buddy->name);
+				   purple_buddy_get_name(buddy));
 }
 
 static void sipe_purple_buddy_remove_from_chat_cb(PurpleBuddy *buddy,
 						  gpointer parameter)
 {
-	SIPE_DEBUG_INFO("sipe_purple_buddy_remove_from_chat_cb: buddy->name=%s",
-			buddy->name);
+	SIPE_DEBUG_INFO("sipe_purple_buddy_remove_from_chat_cb: name '%s'",
+			purple_buddy_get_name(buddy));
 	sipe_core_conf_remove_from(PURPLE_BUDDY_TO_SIPE_CORE_PUBLIC,
 				   parameter,
-				   buddy->name);
+				   purple_buddy_get_name(buddy));
 }
 
 static void sipe_purple_buddy_invite_to_chat_cb(PurpleBuddy *buddy,
 						gpointer parameter)
 {
-	SIPE_DEBUG_INFO("sipe_purple_buddy_invite_to_chat_cb: buddy->name=%s",
-			buddy->name);
+	SIPE_DEBUG_INFO("sipe_purple_buddy_invite_to_chat_cb: name '%s'",
+			purple_buddy_get_name(buddy));
 	sipe_core_chat_invite(PURPLE_BUDDY_TO_SIPE_CORE_PUBLIC,
 			      parameter,
-			      buddy->name);
+			      purple_buddy_get_name(buddy));
 }
 
 static void sipe_purple_buddy_new_chat_cb(PurpleBuddy *buddy,
 					  SIPE_UNUSED_PARAMETER gpointer parameter)
 {
-	SIPE_DEBUG_INFO("sipe_purple_buddy_new_chat_cb: buddy->name=%s",
-			buddy->name);
+	SIPE_DEBUG_INFO("sipe_purple_buddy_new_chat_cb: name '%s'",
+			purple_buddy_get_name(buddy));
 	sipe_core_buddy_new_chat(PURPLE_BUDDY_TO_SIPE_CORE_PUBLIC,
-				 buddy->name);
+				 purple_buddy_get_name(buddy));
 }
 
 static void sipe_purple_buddy_make_call_cb(PurpleBuddy *buddy,
 					   gpointer parameter)
 {
-	SIPE_DEBUG_INFO("sipe_purple_buddy_make_call_cb: buddy->name=%s", buddy->name);
+	SIPE_DEBUG_INFO("sipe_purple_buddy_make_call_cb: name '%s'",
+			purple_buddy_get_name(buddy));
 	sipe_core_buddy_make_call(PURPLE_BUDDY_TO_SIPE_CORE_PUBLIC,
 				  parameter);
 }
@@ -527,10 +565,10 @@ static void sipe_purple_buddy_make_call_cb(PurpleBuddy *buddy,
 static void sipe_purple_buddy_send_email_cb(PurpleBuddy *buddy,
 					    SIPE_UNUSED_PARAMETER gpointer parameter)
 {
-	SIPE_DEBUG_INFO("sipe_purple_buddy_send_email_cb: buddy->name=%s",
-			buddy->name);
+	SIPE_DEBUG_INFO("sipe_purple_buddy_send_email_cb: name '%s'",
+			purple_buddy_get_name(buddy));
 	sipe_core_buddy_send_email(PURPLE_BUDDY_TO_SIPE_CORE_PUBLIC,
-				   buddy->name);
+				   purple_buddy_get_name(buddy));
 }
 
 static void sipe_purple_buddy_access_level_help_cb(PurpleBuddy *buddy,
@@ -540,7 +578,7 @@ static void sipe_purple_buddy_access_level_help_cb(PurpleBuddy *buddy,
 	 * Translators: replace with URL to localized page
 	 * If it doesn't exist copy the original URL
 	 */
-	purple_notify_uri(buddy->account->gc,
+	purple_notify_uri(purple_account_get_connection(purple_buddy_get_account(buddy)),
 			  _("https://sourceforge.net/apps/mediawiki/sipe/index.php?title=Access_Levels"));
 }
 
@@ -555,7 +593,16 @@ static void sipe_purple_ask_access_domain_cb(PurpleConnection *gc,
 					     PurpleRequestFields *fields)
 {
 	const gchar *domain = purple_request_fields_get_string(fields, "access_domain");
-	guint index         = purple_request_fields_get_choice(fields, "container_id");
+	guint index         =
+#if PURPLE_VERSION_CHECK(3,0,0)
+		GPOINTER_TO_UINT(
+#endif
+			purple_request_fields_get_choice(fields, "container_id")
+#if PURPLE_VERSION_CHECK(3,0,0)
+				)
+#endif
+		;
+
 	sipe_core_change_access_level_for_domain(PURPLE_GC_TO_SIPE_CORE_PUBLIC,
 						 domain,
 						 index);
@@ -564,7 +611,8 @@ static void sipe_purple_ask_access_domain_cb(PurpleConnection *gc,
 static void sipe_purple_buddy_add_new_domain_cb(PurpleBuddy *buddy,
 						SIPE_UNUSED_PARAMETER gpointer parameter)
 {
-	PurpleConnection *gc = purple_account_get_connection(buddy->account);
+	PurpleAccount *account = purple_buddy_get_account(buddy);
+	PurpleConnection *gc = purple_account_get_connection(account);
 	PurpleRequestFields *fields;
 	PurpleRequestFieldGroup *g;
 	PurpleRequestField *f;
@@ -582,12 +630,21 @@ static void sipe_purple_buddy_add_new_domain_cb(PurpleBuddy *buddy,
 	f = purple_request_field_choice_new("container_id",
 					    _("Access level"),
 					    0);
+#if PURPLE_VERSION_CHECK(3,0,0)
+	purple_request_field_choice_add(f, _("Personal"), GUINT_TO_POINTER(0));
+	purple_request_field_choice_add(f, _("Team"),     GUINT_TO_POINTER(1));
+	purple_request_field_choice_add(f, _("Company"),  GUINT_TO_POINTER(2));
+	purple_request_field_choice_add(f, _("Public"),   GUINT_TO_POINTER(3));
+	purple_request_field_choice_add(f, _("Blocked"),  GUINT_TO_POINTER(4));
+	purple_request_field_choice_set_default_value(f,  GUINT_TO_POINTER(3));
+#else
 	purple_request_field_choice_add(f, _("Personal")); /* index 0 */
 	purple_request_field_choice_add(f, _("Team"));
 	purple_request_field_choice_add(f, _("Company"));
 	purple_request_field_choice_add(f, _("Public"));
 	purple_request_field_choice_add(f, _("Blocked")); /* index 4 */
 	purple_request_field_choice_set_default_value(f, 3); /* index */
+#endif
 	purple_request_field_set_required(f, TRUE);
 	purple_request_field_group_add_field(g, f);
 
@@ -597,7 +654,12 @@ static void sipe_purple_buddy_add_new_domain_cb(PurpleBuddy *buddy,
 			      _("Add new domain"), NULL, fields,
 			      _("Add"), G_CALLBACK(sipe_purple_ask_access_domain_cb),
 			      _("Cancel"), NULL,
-			      buddy->account, NULL, NULL, gc);
+#if PURPLE_VERSION_CHECK(3,0,0)
+			      purple_request_cpar_from_account(account),
+#else
+			      account, NULL, NULL,
+#endif
+			      gc);
 }
 
 typedef void (*buddy_menu_callback)(PurpleBuddy *buddy,
@@ -662,27 +724,33 @@ static void sipe_purple_buddy_copy_to_cb(PurpleBlistNode *node,
 	PurpleGroup *group;
 	PurpleBuddy *clone;
 
-	g_return_if_fail(PURPLE_BLIST_NODE_IS_BUDDY(node));
+	g_return_if_fail(PURPLE_IS_BUDDY(node));
 
 	sipe_public = PURPLE_BUDDY_TO_SIPE_CORE_PUBLIC;
-	group       = purple_find_group(group_name);
+	group       = purple_blist_find_group(group_name);
 
 	SIPE_DEBUG_INFO("sipe_purple_buddy_copy_to_cb: copying %s to %s",
-			buddy->name, group_name);
+			purple_buddy_get_name(buddy), group_name);
 
-	clone = purple_find_buddy_in_group(buddy->account, buddy->name, group);
+	clone = purple_blist_find_buddy_in_group(purple_buddy_get_account(buddy),
+						 purple_buddy_get_name(buddy),
+						 group);
 	if (!clone) {
 		clone = sipe_backend_buddy_add(sipe_public,
-					       buddy->name,
+					       purple_buddy_get_name(buddy),
+#if PURPLE_VERSION_CHECK(3,0,0)
+					       purple_buddy_get_local_alias(buddy),
+#else
 					       buddy->alias,
-					       group->name);
+#endif
+					       purple_group_get_name(group));
 		if (clone) {
 			const gchar *tmp;
 			const gchar *key;
 			const PurpleStatus *status = purple_presence_get_active_status(purple_buddy_get_presence(buddy));
 
 			tmp = purple_buddy_get_server_alias(buddy);
-			if (tmp) purple_blist_server_alias_buddy(clone, tmp);
+			if (tmp) purple_buddy_set_server_alias(clone, tmp);
 
 			key = buddy_info_property(SIPE_BUDDY_INFO_EMAIL);
 			tmp = purple_blist_node_get_string(&buddy->node, key);
@@ -696,17 +764,18 @@ static void sipe_purple_buddy_copy_to_cb(PurpleBlistNode *node,
 							  TRUE);
 
 			/* update UI */
-			purple_prpl_got_user_status(clone->account,
-						    clone->name,
+			purple_prpl_got_user_status(purple_buddy_get_account(clone),
+						    purple_buddy_get_name(clone),
 						    tmp,
+						    SIPE_PURPLE_STATUS_ATTR_ID_MESSAGE, tmp,
 						    NULL);
 		}
 	}
 
 	if (clone && group)
 		sipe_core_buddy_add(sipe_public,
-				    clone->name,
-				    group->name);
+				    purple_buddy_get_name(clone),
+				    purple_group_get_name(group));
 }
 
 static GList *sipe_purple_copy_to_menu(GList *menu,
@@ -720,16 +789,17 @@ static GList *sipe_purple_copy_to_menu(GList *menu,
 		PurpleGroup *group = (PurpleGroup *)g_node;
 		PurpleMenuAction *act;
 
-		if ((g_node->type != PURPLE_BLIST_GROUP_NODE) ||
-		    (group == gr_parent)                      ||
-		    purple_find_buddy_in_group(buddy->account,
-					       buddy->name,
-					       group))
+		if ((!PURPLE_IS_GROUP(g_node)) ||
+		    (group == gr_parent)       ||
+		    purple_blist_find_buddy_in_group(purple_buddy_get_account(buddy),
+						     purple_buddy_get_name(buddy),
+						     group))
 			continue;
 
 		act = purple_menu_action_new(purple_group_get_name(group),
 					     PURPLE_CALLBACK(sipe_purple_buddy_copy_to_cb),
-					     group->name, NULL);
+					     (gpointer) purple_group_get_name(group),
+					     NULL);
 		menu_groups = g_list_prepend(menu_groups, act);
 	}
 
@@ -747,7 +817,7 @@ GList *sipe_purple_buddy_menu(PurpleBuddy *buddy)
 {
 	struct sipe_core_public *sipe_public = PURPLE_BUDDY_TO_SIPE_CORE_PUBLIC;
 	GList *menu = (GList *) sipe_core_buddy_create_menu(sipe_public,
-							    buddy->name,
+							    purple_buddy_get_name(buddy),
 							    NULL);
 	menu = sipe_purple_copy_to_menu(menu, buddy);
 	return(g_list_reverse(menu));
